@@ -29,55 +29,24 @@ namespace GtkSharp.Generation {
 		
 		ArrayList parms = new ArrayList ();
 		ArrayList special = new ArrayList ();
-		string error_param = null;
-		string user_data_param = null;
-		string destroy_param = null;
 
-		public ManagedCallString (Parameters parms, bool drop_first)
+		public ManagedCallString (Parameters parms)
 		{
-			for (int i = drop_first ? 1 : 0; i < parms.Count; i ++) {
+			for (int i = 1; i < parms.Count; i ++) {
 				Parameter p = parms [i];
-				if (p.IsLength && i > 0 && parms [i-1].IsString) 
+				if (p.IsLength && parms [i-1].IsString)
 					continue;
-				else if (p.Scope == "notified") {
-					user_data_param = parms[i+1].Name;
-					destroy_param = parms[i+2].Name;
-					i += 2;
-				} else if (p.IsUserData && parms.IsHidden (p)) {
-					user_data_param = p.Name;
-					continue;
-				} else if (p is ErrorParameter) {
-					error_param = p.Name;
-					continue;
-				}
 				this.parms.Add (p);
 
-				if (p.PassAs != String.Empty && (p.Name != p.FromNative (p.Name)))
-					this.special.Add (true);
-				else if (p.Generatable is CallbackGen)
+				IGeneratable igen = p.Generatable;
+				if ((igen is StructGen || p.PassAs != "") &&
+				    (p.Name != p.FromNative (p.Name)))
 					this.special.Add (true);
 				else
 					this.special.Add (false);
 			}
 		}
 
-		public bool HasOutParam {
-			get {
-				foreach (Parameter p in parms) {
-					if (p.PassAs == "out")
-						return true;
-				}
-				return false;
-			}
-		}
-
-		public string Unconditional (string indent) {
-			string ret = "";
-			if (error_param != null)
-				ret = indent + error_param + " = IntPtr.Zero;\n";
-			return ret;
-		}
-
 		public string Setup (string indent)
 		{
 			string ret = "";
@@ -89,19 +58,10 @@ namespace GtkSharp.Generation {
 				Parameter p = parms [i] as Parameter;
 				IGeneratable igen = p.Generatable;
 
-				if (igen is CallbackGen) {
-					if (user_data_param == null)
-						ret += indent + String.Format ("{0} {1}_invoker = new {0} ({1});\n", (igen as CallbackGen).InvokerName, p.Name);
-					else if (destroy_param == null)
-						ret += indent + String.Format ("{0} {1}_invoker = new {0} ({1}, {2});\n", (igen as CallbackGen).InvokerName, p.Name, user_data_param);
-					else
-						ret += indent + String.Format ("{0} {1}_invoker = new {0} ({1}, {2}, {3});\n", (igen as CallbackGen).InvokerName, p.Name, user_data_param, destroy_param);
-				} else {
-					ret += indent + igen.QualifiedName + " my" + p.Name;
-					if (p.PassAs == "ref")
-						ret += " = " + p.FromNative (p.Name);
-					ret += ";\n";
-				}
+				ret += indent + igen.QualifiedName + " my" + p.Name;
+				if (igen is StructGen || p.PassAs == "ref")
+					ret += " = " + p.FromNative (p.Name);
+				ret += ";\n";
 			}
 
 			return ret;
@@ -117,10 +77,7 @@ namespace GtkSharp.Generation {
 			for (int i = 0; i < parms.Count; i ++) {
 				Parameter p = parms [i] as Parameter;
 				result [i] = p.PassAs == "" ? "" : p.PassAs + " ";
-				if (p.Generatable is CallbackGen)
-					result [i] += p.Name + "_invoker.Handler";
-				else
-					result [i] += ((bool)special[i]) ? "my" + p.Name : p.FromNative (p.Name);
+				result [i] += ((bool)special[i]) ? "my" + p.Name : p.FromNative (p.Name);
 			}
 
 			return String.Join (", ", result);
@@ -137,11 +94,9 @@ namespace GtkSharp.Generation {
 				Parameter p = parms [i] as Parameter;
 				IGeneratable igen = p.Generatable;
 
-				if (igen is CallbackGen)
-					continue;
-				else if (igen is StructBase)
+				if (igen is StructBase)
 					ret += indent + String.Format ("if ({0} != IntPtr.Zero) System.Runtime.InteropServices.Marshal.StructureToPtr (my{0}, {0}, false);\n", p.Name);
-				else if (!(igen is ByRefGen))
+				else
 					ret += indent + p.Name + " = " + igen.ToNativeReturn ("my" + p.Name) + ";\n";
 			}
 
