@@ -26,7 +26,7 @@ namespace GLib {
 
 	public class List : ListBase {
 
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libglib-2.0-0.dll")]
 		static extern IntPtr g_list_copy (IntPtr l);
 		
 		public override object Clone ()
@@ -34,7 +34,23 @@ namespace GLib {
 			return new List (g_list_copy (Handle));
 		}
 		
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("glibsharpglue-2")]
+		static extern IntPtr gtksharp_list_get_data (IntPtr l);
+		
+		internal override IntPtr GetData (IntPtr current)
+		{
+			return gtksharp_list_get_data (current);
+		}
+
+		[DllImport("glibsharpglue-2")]
+		static extern IntPtr gtksharp_list_get_next (IntPtr l);
+		
+		internal override IntPtr Next (IntPtr current)
+		{
+			return gtksharp_list_get_next (current);
+		}
+
+		[DllImport("libglib-2.0-0.dll")]
 		static extern int g_list_length (IntPtr l);
 		
 		internal override int Length (IntPtr list)
@@ -42,7 +58,7 @@ namespace GLib {
 			return g_list_length (list);
 		}
 		
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libglib-2.0-0.dll")]
 		static extern void g_list_free(IntPtr l);
 
 		internal override void Free (IntPtr list)
@@ -51,7 +67,7 @@ namespace GLib {
 				g_list_free (list);
 		}
 
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libglib-2.0-0.dll")]
 		static extern IntPtr g_list_append (IntPtr l, IntPtr raw);
 
 		internal override IntPtr Append (IntPtr list, IntPtr raw)
@@ -59,7 +75,7 @@ namespace GLib {
 			return g_list_append (list, raw);
 		}
 
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libglib-2.0-0.dll")]
 		static extern IntPtr g_list_prepend (IntPtr l, IntPtr raw);
 
 		internal override IntPtr Prepend (IntPtr list, IntPtr raw)
@@ -67,7 +83,7 @@ namespace GLib {
 			return g_list_prepend (list, raw);
 		}
 
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libglib-2.0-0.dll")]
 	        static extern IntPtr g_list_nth_data (IntPtr l, uint n);
 
 		internal override IntPtr NthData (uint n)
@@ -75,28 +91,13 @@ namespace GLib {
 			return g_list_nth_data (Handle, n);
 		}
 
-		[DllImport ("libglib-2.0-0.dll", CallingConvention = CallingConvention.Cdecl)]
-		static extern void g_free (IntPtr item);
-
 		public List (IntPtr raw) : this (raw, null) {}
 
 		public List (System.Type element_type) : this (IntPtr.Zero, element_type) {}
 
 		public List (IntPtr raw, System.Type element_type) : this (raw, element_type, false, false) {}
 
-		public List (IntPtr raw, System.Type element_type, bool owned, bool elements_owned) : base (raw, element_type, owned, elements_owned, g_free) { }
+		public List (IntPtr raw, System.Type element_type, bool owned, bool elements_owned) : base (raw, element_type, owned, elements_owned) {}
 
-		public List (IntPtr raw, System.Type element_type, bool owned, bool elements_owned, ListElementFree free_func) : base (raw, element_type, owned, elements_owned, free_func) {}
-
-		public List (object[] elements, System.Type element_type, bool owned, bool elements_owned) : this (IntPtr.Zero, element_type, owned, elements_owned) 
-		{
-			foreach (object o in elements)
-				Append (o);
-		}
-		public List (Array elements, System.Type element_type, bool owned, bool elements_owned) : this (IntPtr.Zero, element_type, owned, elements_owned) 
-		{
-			foreach (object o in elements)
-				Append (o);
-		}
 	}
 }
