@@ -26,7 +26,7 @@ namespace GLib {
 
 	public class SList : ListBase {
 
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libglib-2.0-0.dll")]
 		static extern IntPtr g_slist_copy (IntPtr l);
 		
 		public override object Clone ()
@@ -34,7 +34,23 @@ namespace GLib {
 			return new SList (g_slist_copy (Handle));
 		}
 		
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("glibsharpglue-2")]
+		static extern IntPtr gtksharp_slist_get_data (IntPtr l);
+		
+		internal override IntPtr GetData (IntPtr current)
+		{
+			return gtksharp_slist_get_data (current);
+		}
+
+		[DllImport("glibsharpglue-2")]
+		static extern IntPtr gtksharp_slist_get_next (IntPtr l);
+		
+		internal override IntPtr Next (IntPtr current)
+		{
+			return gtksharp_slist_get_next (current);
+		}
+
+		[DllImport("libglib-2.0-0.dll")]
 		static extern int g_slist_length (IntPtr l);
 		
 		internal override int Length (IntPtr list)
@@ -42,7 +58,7 @@ namespace GLib {
 			return g_slist_length (list);
 		}
 		
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libglib-2.0-0.dll")]
 		static extern void g_slist_free(IntPtr l);
 
 		internal override void Free (IntPtr list)
@@ -51,7 +67,7 @@ namespace GLib {
 				g_slist_free (list);
 		}
 
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libglib-2.0-0.dll")]
 		static extern IntPtr g_slist_append (IntPtr l, IntPtr raw);
 
 		internal override IntPtr Append (IntPtr list, IntPtr raw)
@@ -59,7 +75,7 @@ namespace GLib {
 			return g_slist_append (list, raw);
 		}
 
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libglib-2.0-0.dll")]
 		static extern IntPtr g_slist_prepend (IntPtr l, IntPtr raw);
 
 		internal override IntPtr Prepend (IntPtr list, IntPtr raw)
@@ -68,7 +84,7 @@ namespace GLib {
 		}
 
 
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libglib-2.0-0.dll")]
 	        static extern IntPtr g_slist_nth_data (IntPtr l, uint n);
 
 		internal override IntPtr NthData (uint n)
@@ -76,31 +92,12 @@ namespace GLib {
 			return g_slist_nth_data (Handle, n);
 		}
 
-
-		[DllImport ("libglib-2.0-0.dll", CallingConvention = CallingConvention.Cdecl)]
-		static extern void g_free (IntPtr item);
-
 		public SList (IntPtr raw) : this (raw, null) {}
 
 		public SList (System.Type element_type) : this (IntPtr.Zero, element_type) {}
 
 		public SList (IntPtr raw, System.Type element_type) : this (raw, element_type, false, false) {}
 
-		public SList (IntPtr raw, System.Type element_type, bool owned, bool elements_owned) : base (raw, element_type, owned, elements_owned, g_free) { }
-
-		public SList (IntPtr raw, System.Type element_type, bool owned, bool elements_owned, ListElementFree free_func) : base (raw, element_type, owned, elements_owned, free_func) { }
-
-
-		public SList (object[] members, System.Type element_type, bool owned, bool elements_owned) : this (IntPtr.Zero, element_type, owned, elements_owned)
-		{
-			foreach (object o in members)
-				Append (o);
-		}
-
-		public SList (Array members, System.Type element_type, bool owned, bool elements_owned) : this (IntPtr.Zero, element_type, owned, elements_owned)
-		{
-			foreach (object o in members)
-				Append (o);
-		}
+		public SList (IntPtr raw, System.Type element_type, bool owned, bool elements_owned) : base (raw, element_type, false, false) {}
 	}
 }
